@@ -35,12 +35,39 @@ use crate::logging::ConfigLogging;
 use crate::pagination::ResultsPage;
 use crate::server::{HttpServer, HttpServerStarter, ServerContext};
 
-/**
- * List of allowed HTTP headers in responses.  This is used to make sure we
- * don't leak headers unexpectedly.
- */
-const ALLOWED_HEADER_NAMES: [&str; 4] =
-    ["content-length", "content-type", "date", "x-request-id"];
+enum AllowedValue<'a> {
+    Any,
+    OneOf(&'a [&'a str]),
+}
+
+struct AllowedHeader<'a> {
+    name: &'a str,
+    value: AllowedValue<'a>,
+}
+
+impl<'a> AllowedHeader<'a> {
+    const fn new(name: &'a str) -> Self {
+        Self { name, value: AllowedValue::Any }
+    }
+}
+
+pub const TEST_HEADER_1: &str = "x-dropshot-test-header-1";
+pub const TEST_HEADER_2: &str = "x-dropshot-test-header-2";
+
+// List of allowed HTTP headers in responses.
+// Used to make sure we don't leak headers unexpectedly.
+const ALLOWED_HEADERS: [AllowedHeader<'static>; 7] = [
+    AllowedHeader::new("content-length"),
+    AllowedHeader::new("content-type"),
+    AllowedHeader::new("date"),
+    AllowedHeader::new("x-request-id"),
+    AllowedHeader {
+        name: "transfer-encoding",
+        value: AllowedValue::OneOf(&["chunked"]),
+    },
+    AllowedHeader::new(TEST_HEADER_1),
+    AllowedHeader::new(TEST_HEADER_2),
+];
 
 /**
  * ClientTestContext encapsulates several facilities associated with using an
@@ -48,11 +75,11 @@ const ALLOWED_HEADER_NAMES: [&str; 4] =
  */
 pub struct ClientTestContext {
     /** actual bind address of the HTTP server under test */
-    bind_address: SocketAddr,
+    pub bind_address: SocketAddr,
     /** HTTP client, used for making requests against the test server */
-    client: Client<HttpConnector>,
+    pub client: Client<HttpConnector>,
     /** logger for the test suite HTTP client */
-    client_log: Logger,
+    pub client_log: Logger,
 }
 
 impl ClientTestContext {
@@ -199,11 +226,21 @@ impl ClientTestContext {
          * statically-defined above.
          */
         let headers = response.headers();
-        for header_name in headers.keys() {
+        for (header_name, header_value) in headers {
             let mut okay = false;
-            for allowed_name in ALLOWED_HEADER_NAMES.iter() {
-                if header_name == allowed_name {
-                    okay = true;
+            for allowed_header in ALLOWED_HEADERS.iter() {
+                if header_name == allowed_header.name {
+                    match allowed_header.value {
+                        AllowedValue::Any => {
+                            okay = true;
+                        }
+                        AllowedValue::OneOf(allowed_values) => {
+                            let header = header_value
+                                .to_str()
+                                .expect("Cannot turn header value to string");
+                            okay = allowed_values.contains(&header);
+                        }
+                    }
                     break;
                 }
             }
@@ -364,11 +401,7 @@ impl LogContext {
          * environment variable or other flag.
          */
         let (log_path, log_config) = match initial_config_logging {
-            ConfigLogging::File {
-                level,
-                path: dummy_path,
-                if_exists,
-            } => {
+            ConfigLogging::File { level, path: dummy_path, if_exists } => {
                 assert_eq!(
                     dummy_path, "UNUSED",
                     "for test suite logging configuration, when mode = \
@@ -379,20 +412,20 @@ impl LogContext {
                 let new_path = log_file_for_test(test_name);
                 let new_path_str = new_path.as_path().display().to_string();
                 eprintln!("log file: {:?}", new_path_str);
-                (Some(new_path), ConfigLogging::File {
-                    level: level.clone(),
-                    path: new_path_str,
-                    if_exists: if_exists.clone(),
-                })
+                (
+                    Some(new_path),
+                    ConfigLogging::File {
+                        level: level.clone(),
+                        path: new_path_str,
+                        if_exists: if_exists.clone(),
+                    },
+                )
             }
             other_config => (None, other_config.clone()),
         };
 
         let log = log_config.to_logger(test_name).unwrap();
-        LogContext {
-            log,
-            log_path,
-        }
+        LogContext { log, log_path }
     }
 
     /**
@@ -452,12 +485,7 @@ impl<Context: ServerContext> TestContext<Context> {
         let client_log = log.new(o!("http_client" => "dropshot test suite"));
         let client_testctx = ClientTestContext::new(server_addr, client_log);
 
-        TestContext {
-            client_testctx,
-            server,
-            log,
-            log_context,
-        }
+        TestContext { client_testctx, server, log, log_context }
     }
 
     /**
@@ -854,72 +882,93 @@ mod test {
         /* Test case: nothing to check. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: None,
-            pid: None,
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: None,
+                pid: None,
+                v: None,
+            },
+        );
 
         /* Test case: check name, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: Some("n1".to_string()),
-            hostname: None,
-            pid: None,
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: Some("n1".to_string()),
+                hostname: None,
+                pid: None,
+                v: None,
+            },
+        );
 
         /* Test case: check hostname, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: Some("h1".to_string()),
-            pid: None,
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: Some("h1".to_string()),
+                pid: None,
+                v: None,
+            },
+        );
 
         /* Test case: check pid, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: None,
-            pid: Some(1),
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: None,
+                pid: Some(1),
+                v: None,
+            },
+        );
 
         /* Test case: check hostname, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: None,
-            pid: None,
-            v: Some(0),
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: None,
+                pid: None,
+                v: Some(0),
+            },
+        );
 
         /* Test case: check all, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: Some("n1".to_string()),
-            hostname: Some("h1".to_string()),
-            pid: Some(1),
-            v: Some(0),
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: Some("n1".to_string()),
+                hostname: Some("h1".to_string()),
+                pid: Some(1),
+                v: Some(0),
+            },
+        );
 
         /* Test case: check multiple records, no problem. */
         let records: Vec<&BunyanLogRecord> = vec![&r1, &r2];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: Some("n1".to_string()),
-            hostname: None,
-            pid: Some(1),
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: Some("n1".to_string()),
+                hostname: None,
+                pid: Some(1),
+                v: None,
+            },
+        );
     }
 
     /*
@@ -932,12 +981,15 @@ mod test {
         let r1 = make_dummy_record();
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: Some("n2".to_string()),
-            hostname: None,
-            pid: None,
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: Some("n2".to_string()),
+                hostname: None,
+                pid: None,
+                v: None,
+            },
+        );
     }
 
     #[test]
@@ -946,12 +998,15 @@ mod test {
         let r1 = make_dummy_record();
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: Some("h2".to_string()),
-            pid: None,
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: Some("h2".to_string()),
+                pid: None,
+                v: None,
+            },
+        );
     }
 
     #[test]
@@ -960,12 +1015,15 @@ mod test {
         let r1 = make_dummy_record();
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: None,
-            pid: Some(2),
-            v: None,
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: None,
+                pid: Some(2),
+                v: None,
+            },
+        );
     }
 
     #[test]
@@ -974,12 +1032,15 @@ mod test {
         let r1 = make_dummy_record();
         let records: Vec<&BunyanLogRecord> = vec![&r1];
         let iter = records.iter().map(|x| *x);
-        verify_bunyan_records(iter, &BunyanLogRecordSpec {
-            name: None,
-            hostname: None,
-            pid: None,
-            v: Some(1),
-        });
+        verify_bunyan_records(
+            iter,
+            &BunyanLogRecordSpec {
+                name: None,
+                hostname: None,
+                pid: None,
+                v: Some(1),
+            },
+        );
     }
 
     /*
