@@ -19,10 +19,14 @@ use dropshot::endpoint;
 use dropshot::test_util::object_delete;
 use dropshot::test_util::read_json;
 use dropshot::test_util::read_string;
+use dropshot::test_util::TEST_HEADER_1;
+use dropshot::test_util::TEST_HEADER_2;
 use dropshot::ApiDescription;
 use dropshot::HttpError;
 use dropshot::HttpResponseDeleted;
+use dropshot::HttpResponseHeaders;
 use dropshot::HttpResponseOk;
+use dropshot::HttpResponseUpdatedNoContent;
 use dropshot::Path;
 use dropshot::Query;
 use dropshot::RequestContext;
@@ -39,10 +43,9 @@ use serde::Serialize;
 use std::sync::Arc;
 use uuid::Uuid;
 
-#[macro_use]
 extern crate slog;
 
-mod common;
+pub mod common;
 
 fn demo_api() -> ApiDescription<usize> {
     let mut api = ApiDescription::<usize>::new();
@@ -55,6 +58,7 @@ fn demo_api() -> ApiDescription<usize> {
     api.register(demo_handler_path_param_u32).unwrap();
     api.register(demo_handler_untyped_body).unwrap();
     api.register(demo_handler_delete).unwrap();
+    api.register(demo_handler_headers).unwrap();
 
     /*
      * We don't need to exhaustively test these cases, as they're tested by unit
@@ -201,10 +205,7 @@ async fn test_demo2json() {
     let testctx = common::test_setup("demo2json", api);
 
     /* Test case: optional field */
-    let input = DemoJsonBody {
-        test1: "bar".to_string(),
-        test2: None,
-    };
+    let input = DemoJsonBody { test1: "bar".to_string(), test2: None };
     let mut response = testctx
         .client_testctx
         .make_request(
@@ -220,10 +221,7 @@ async fn test_demo2json() {
     assert_eq!(json.test2, None);
 
     /* Test case: both fields populated */
-    let input = DemoJsonBody {
-        test1: "bar".to_string(),
-        test2: Some(15),
-    };
+    let input = DemoJsonBody { test1: "bar".to_string(), test2: Some(15) };
     let mut response = testctx
         .client_testctx
         .make_request(
@@ -295,10 +293,7 @@ async fn test_demo3json() {
     let testctx = common::test_setup("demo3json", api);
 
     /* Test case: everything filled in. */
-    let json_input = DemoJsonBody {
-        test1: "bart".to_string(),
-        test2: Some(0),
-    };
+    let json_input = DemoJsonBody { test1: "bart".to_string(), test2: Some(0) };
 
     let mut response = testctx
         .client_testctx
@@ -317,10 +312,7 @@ async fn test_demo3json() {
     assert_eq!(json.query.test2.unwrap(), 2);
 
     /* Test case: error parsing query */
-    let json_input = DemoJsonBody {
-        test1: "bart".to_string(),
-        test2: Some(0),
-    };
+    let json_input = DemoJsonBody { test1: "bart".to_string(), test2: Some(0) };
     let error = testctx
         .client_testctx
         .make_request(
@@ -619,6 +611,41 @@ async fn test_delete_request() {
     testctx.teardown().await;
 }
 
+/*
+ * Test response headers
+ */
+#[tokio::test]
+async fn test_header_request() {
+    let api = demo_api();
+    let testctx = common::test_setup("test_header_request", api);
+    let response = testctx
+        .client_testctx
+        .make_request(
+            Method::GET,
+            "/testing/headers",
+            None as Option<()>,
+            StatusCode::NO_CONTENT,
+        )
+        .await
+        .expect("expected success");
+
+    let headers = response
+        .headers()
+        .get_all(TEST_HEADER_1)
+        .iter()
+        .map(|v| v.to_str().unwrap())
+        .collect::<Vec<_>>();
+    assert_eq!(headers, vec!["howdy"]);
+
+    let headers = response
+        .headers()
+        .get_all(TEST_HEADER_2)
+        .iter()
+        .map(|v| v.to_str().unwrap())
+        .collect::<Vec<_>>();
+    assert_eq!(headers, vec!["hi", "howdy"]);
+}
+
 /*
  * Demo handler functions
  */
@@ -681,10 +708,8 @@ async fn demo_handler_args_3(
     query: Query<DemoQueryArgs>,
     json: TypedBody<DemoJsonBody>,
 ) -> Result<Response<Body>, HttpError> {
-    let combined = DemoJsonAndQuery {
-        query: query.into_inner(),
-        json: json.into_inner(),
-    };
+    let combined =
+        DemoJsonAndQuery { query: query.into_inner(), json: json.into_inner() };
     http_echo(&combined)
 }
 
@@ -761,10 +786,7 @@ async fn demo_handler_untyped_body(
         None
     };
 
-    Ok(HttpResponseOk(DemoUntyped {
-        nbytes,
-        as_utf8,
-    }))
+    Ok(HttpResponseOk(DemoUntyped { nbytes, as_utf8 }))
 }
 
 #[derive(Deserialize, Serialize, JsonSchema)]
@@ -792,6 +814,25 @@ async fn demo_handler_delete(
     Ok(HttpResponseDeleted())
 }
 
+#[endpoint {
+    method = GET,
+    path = "/testing/headers",
+}]
+async fn demo_handler_headers(
+    _rqctx: RequestCtx,
+) -> Result<HttpResponseHeaders<HttpResponseUpdatedNoContent>, HttpError> {
+    let mut response =
+        HttpResponseHeaders::new_unnamed(HttpResponseUpdatedNoContent());
+    let headers = response.headers_mut();
+    headers.insert(TEST_HEADER_1, http::header::HeaderValue::from_static("hi"));
+    headers
+        .insert(TEST_HEADER_1, http::header::HeaderValue::from_static("howdy"));
+    headers.append(TEST_HEADER_2, http::header::HeaderValue::from_static("hi"));
+    headers
+        .append(TEST_HEADER_2, http::header::HeaderValue::from_static("howdy"));
+    Ok(response)
+}
+
 fn http_echo<T: Serialize>(t: &T) -> Result<Response<Body>, HttpError> {
     Ok(Response::builder()
         .header(http::header::CONTENT_TYPE, CONTENT_TYPE_JSON)
